<?php

class I18nFrontend {
  
  private static $initialized = false;
  private static $languages = null;
  
  public static function getDefaultLanguage() {
    require_once(GSPLUGINPATH.'i18n_base/basic.class.php');
    return I18nBasic::getProperty(I18N_PROP_DEFAULT_LANGUAGE, I18N_DEFAULT_LANGUAGE);
  }
  
  public static function getLanguages() {
    if (!self::$languages) {
      self::$languages = array();
      if (isset($_GET[I18N_LANGUAGE_PARAM])) {
        self::$languages[] = $_GET[I18N_LANGUAGE_PARAM];
      }
      if (isset($_SESSION[I18N_LANGUAGE_KEY]) && !in_array($_SESSION[I18N_LANGUAGE_KEY],self::$languages)) {
        self::$languages[] = $_SESSION[I18N_LANGUAGE_KEY];
      }
      $httplanguages = explode(",", @$_SERVER['HTTP_ACCEPT_LANGUAGE']);
      foreach ($httplanguages as $language) {
        $language = substr($language,0,2);
        if (!in_array($language,self::$languages)) self::$languages[] = $language;
      }
      $deflang = self::getDefaultLanguage();
      if (!in_array($deflang, self::$languages)) self::$languages[] = $deflang;
    }
    return self::$languages;
  }
  
  public static function init() {
    if (self::$initialized) return;
    @session_start();
    if (isset($_GET[I18N_SET_LANGUAGE_PARAM])) {
      $_SESSION[I18N_LANGUAGE_KEY] = $_GET[I18N_SET_LANGUAGE_PARAM];
    }
    $languages = self::getLanguages();
    self::$initialized = true;
    foreach ($languages as $language) {
      if (self::load($language)) return;
    }
  }
  
  public static function getPageData($slug) {
    $languages = self::getLanguages();
    $deflang = self::getDefaultLanguage();
    foreach ($languages as $lang) {
      $file = GSDATAPAGESPATH . $slug . ($lang == $deflang ? '' : '_' . $lang) .'.xml';
      if (file_exists($file)) {
        return getXML($file);
      }
    }
    return null;
  }
  
  public static function outputContent($slug, $force=false) {
    $data = null;
    if ($force) {
      $file = GSDATAPAGESPATH . $slug . '.xml';
      if (file_exists($file)) $data = getXML($file);
    } else {
      $data = self::getPageData($slug);
    }
    if ($data) {
      $content = $data->content;
      $content = stripslashes(htmlspecialchars_decode($content, ENT_QUOTES));
      $content = exec_filter('content',$content);
      echo $content;
      return true;
    } else {
      return false;
    }
  }
  
  public static function getComponent($id) {
    if (file_exists(GSDATAOTHERPATH.'components.xml')) {
      $deflang = self::getDefaultLanguage();
      $data = getXML(GSDATAOTHERPATH.'components.xml');
      $components = $data->item;
      if (count($components) != 0) {
        $languages = self::getLanguages();
        $isV3 = function_exists('get_site_version'); 
        $sep = $isV3 ? '_' : '';
        foreach ($languages as $lang) {
          $slug = $id . ($lang == $deflang ? '' : $sep . $lang);
          foreach ($components as $component) {
            if ($slug == $component->slug) { 
              return stripslashes(htmlspecialchars_decode($component->value, ENT_QUOTES)); 
            }
          }
        }
      }
    }
    return null;
  }
  
  public static function outputComponent($id, $arguments=null) {
    global $args;
    $component = self::getComponent($id);
    if (isset($args)) $saved_args = $args;
    $args = $arguments;
    if ($component) eval("?>" . $component . "<?php ");
    if (isset($saved_args)) $args = $saved_args; else unset($args); 
    return $component ? true : false;
  }
  
  public static function getURL($slug, $slugparent, $language=null, $type='full') {
    global $url, $parent, $PERMALINK, $PERMALINK_ORIG;
    if (!$slug) {
      $slug = @$url;
      $slugparent = @$parent;
    }
    if ($language && @strpos(@$PERMALINK_ORIG,'%language%') !== false) {
      $PERMALINK_SAVED = $PERMALINK;
      $PERMALINK = str_replace('%language%', $language, $PERMALINK_ORIG);
      $u = find_url($slug, $slugparent, $type);
      $PERMALINK = $PERMALINK_SAVED;
    } else { 
      $u = find_url($slug, $slugparent, $type);
      if ($language) {
        $u .= (strpos($u,'?') !== false ? '&' : '?') . 'lang=' . $language;
      }
    }
    return $u;
  }

  public static function getLandURL($language=null) {
    global $PERMALINK_ORIG;
    if (!$language) $language = self::getDefaultLanguage();
    if (@strpos(@$PERMALINK_ORIG,'%language%') !== false) {
      return self::getURL(null,null,$language); 
    } else {
      return self::getCurrentURLWithParameter('lang',$language);
    }
  }

  public static function getSetLangURL($language) {
    global $PERMALINK_ORIG;
    if (@strpos(@$PERMALINK_ORIG,'%language%') !== false) {
      return self::getURL(null,null,$language); // no setlang parameter supported for URLs with language
    } else {
      return self::getCurrentURLWithParameter('setlang',$language);
    }
  }

  public static function outputLinkTo($slug) {
    $data = self::getPageData($slug);
    if (!$data) return false;
    echo '<a href="'.find_url($slug,(string) $data->parent).'">'.stripslashes((string) $data->title).'</a>';
    return true;
  }

  // like get_header, but tags beginning with _ are ignored and the language is appended to the canonical URL
  public static function outputHeader($full=true) {
    global $metad, $metak, $title, $content, $url, $parent, $language;
    include(GSADMININCPATH.'configuration.php');
    if ($metad != '') {
      $description = stripslashes(htmlspecialchars_decode($metad, ENT_QUOTES));
    } else {
      if (function_exists('mb_substr')) { 
        $description = trim(mb_substr(strip_tags(stripslashes(htmlspecialchars_decode($content, ENT_QUOTES))), 0, 160));
      } else {
        $description = trim(substr(strip_tags(stripslashes(htmlspecialchars_decode($content, ENT_QUOTES))), 0, 160));
      }
      $description = str_replace('"','', $description);
      $description = str_replace("'",'', $description);
      $description = preg_replace('/\n/', " ", $description);
      $description = preg_replace('/\r/', " ", $description);
      $description = preg_replace('/\t/', " ", $description);
      $description = preg_replace('/ +/', " ", $description);
    }
    $keywords = array();
    $tags = preg_split("/\s*,\s*/", stripslashes(htmlspecialchars_decode($metak, ENT_QUOTES)));
    if (count($tags) > 0) foreach ($tags as $tag) if (substr(trim($tag),0,1) != '_') $keywords[] = trim($tag);
    
    echo '<meta name="description" content="'.htmlspecialchars(trim($description)).'" />'."\n";
    echo '<meta name="keywords" content="'.htmlspecialchars(implode(', ',$keywords)).'" />'."\n";
    if ($full) {
      echo '<meta name="generator" content="'.$site_full_name.'" />'."\n";
      echo '<link rel="canonical" href="'.find_i18n_url($url,$parent,$language).'" />'."\n";
    }
    exec_action('theme-header');
  }

  private static function load($lang) {
    global $language, $data_index_orig;
    global $data_index, $id, $title, $date, $metak, $metad, $content, $parent, $template_file, $private;
    global $PERMALINK, $PERMALINK_ORIG;
    $deflang = self::getDefaultLanguage();
    if ($lang == $deflang) {
      $language = $deflang;
    } else { 
      // load language
      $file = GSDATAPAGESPATH . $id . '_' . $lang .'.xml';
      if (file_exists($file)) {
        $data = getXML($file);
        $title = $data->title;
        $date = $data->pubDate;
        $metak = $data->meta;
        $metad = $data->metad;
        // $url = $data->url;  // URL stays the same!
        $content = $data->content;
        $parent = $data->parent;
        // $template_file = $data->template;  // template stays the same
        $private = $data->private;
        // language
        $language = $lang;
        $data_index_orig = $data_index;
        $data_index = $data;
      } else {
        return false;
      }
    }
    if (@$PERMALINK) {
      $PERMALINK_ORIG = $PERMALINK;
      $PERMALINK = str_replace('%language%', @$_GET['lang']?$_GET['lang']:$language, $PERMALINK);
    }
    return true;
  }
  
  private static function getCurrentURLWithParameter($name, $value) {
    $pos = strpos($_SERVER["REQUEST_URI"],'?');
    if ($pos !== false) {
      $url = substr($_SERVER["REQUEST_URI"],0,$pos+1);
      $params = preg_split('/&/', substr($_SERVER["REQUEST_URI"], $pos+1));
      foreach ($params as $param) {
        if (substr($param,0,strlen($name)+1) != $name.'=') $url .= $param.'&';
      }
    } else {
      $url = $_SERVER["REQUEST_URI"].'?';
    }
    if ($value != null) $url .= $name.'='.urlencode($value); else $url = substr($url,0,strlen($url)-1);
    return $url;
  }
 
}